@@ -176,7 +176,6 @@ cocoa_menu_item_update_submenu (_GNSMenuItem *cocoa_item,
       cocoa_submenu = [[[NSMenu alloc] initWithTitle: title] autorelease];
 
       [cocoa_submenu setDelegate: delegate];
-      [cocoa_submenu setAutoenablesItems: NO];
       cocoa_menu_connect (submenu, cocoa_submenu);
       /* connect the new nsmenu to the passed-in item (which lives in
          the parent nsmenu)
@@ -673,7 +672,7 @@ cocoa_menu_item_add_submenu (GtkMenuShell *menu_shell,
 	 */
 	continue;
 
-#ifndef HAVE_GTK_34
+#if ! GTK_CHECK_VERSION (3, 4, 0)
       if (GTK_IS_TEAROFF_MENU_ITEM (menu_item))
         /*Don't want tearoff items at all */
         continue;
@@ -692,14 +691,17 @@ cocoa_menu_item_add_submenu (GtkMenuShell *menu_shell,
       _GNSMenuItem *item = (_GNSMenuItem*)[cocoa_menu itemAtIndex: index];
       if ([item isSeparatorItem] &&
 	  (last_cocoa_item == nil || [last_cocoa_item isSeparatorItem]))
+	{
 	  [cocoa_menu removeItem: item];
+	  continue;
+	}
       if (([item respondsToSelector: @selector (isMarked)]) && [item isMarked])
 	[cocoa_menu removeItem: item];
       else if (![item isHidden])
 	last_cocoa_item = item;
     }
   /* Finally make sure that the last item isn't a separator. */
-  if ([last_cocoa_item isSeparatorItem])
+  if (last_cocoa_item != nil && [last_cocoa_item isSeparatorItem])
     [cocoa_menu removeItem: last_cocoa_item];
 
   g_list_free (children);
