@@ -31,6 +31,7 @@
 #import "GNSMenuBar.h"
 #import "GNSMenuItem.h"
 
+#include <config.h>
 #include "gtkosxapplication.h"
 #include "gtkosxapplicationprivate.h"
 #include "cocoa_menu_item.h"
@@ -38,6 +39,11 @@
 #include "getlabel.h"
 #include "gtkosx-image.h"
 
+#ifdef G_LOG_DOMAIN
+#undef G_LOG_DOMAIN
+#endif
+#define G_LOG_DOMAIN "gtkosxapplication"
+
 /* This is a private function in libgdk; we need to have is so that we
    can force new windows onto the Window menu */
 extern NSWindow* gdk_quartz_window_get_nswindow (GdkWindow*);
@@ -160,6 +166,35 @@ add_to_menubar (GtkosxApplication *self, NSMenu *menu, int pos)
   return dummyItem;
 }
 
+static NSString*
+get_application_name (void)
+{
+  NSString *appname = nil;
+  NSBundle *bundle = [NSBundle mainBundle];
+  if ([bundle bundleIdentifier])
+    {
+
+      appname = [[bundle infoDictionary]
+		 objectForKey: (NSString*)kCFBundleNameKey];
+      if (appname == nil)
+	appname = [[bundle infoDictionary]
+		   objectForKey: @"CFBundleExecutable"];
+      if (appname == nil)
+	{
+	  NSString *bundlep = [bundle bundlePath];
+	  appname =  [[NSFileManager defaultManager]
+		      displayNameAtPath: bundlep];
+	  g_message ("[get_application_name]: no bundle name key in Info.plist\n");
+	}
+    }
+  else
+    {
+      NSString *exepath = [bundle executablePath];
+      appname =  [[NSFileManager defaultManager] displayNameAtPath: exepath];
+    }
+  return appname;
+}
+
 /*
  * create_apple_menu:
  * @self: The GtkosxApplication object.
@@ -174,6 +209,16 @@ add_to_menubar (GtkosxApplication *self, NSMenu *menu, int pos)
  * be GtkosxApplication.strings, and must be installed in lang.proj in
  * the application bundle's Resources directory.
  *
+ * Several menu items have the application name appended, in accord
+ * with Apple's practice. When the app is bundled, it will look for
+ * the Info.plist keys CFBundleName and CFBundleExecutable. If neither
+ * exists (unlikely, since the app won't launch if CFBundleExecutable
+ * isn't set), it will fall back to the name of the bundle, including
+ * the ".app" extension if the user has checked "display all
+ * extensions" in Finder>Preferences>Advanced. When not bundled, the
+ * application name will be the executable name. This will generally
+ * match what LaunchServices names the app menu.
+ *
  * Returns: A pointer to the menu item.
  */
 static _GNSMenuItem*
@@ -186,9 +231,7 @@ create_apple_menu (GtkosxApplication *self)
 						@"GtkOSXApplication",
 						@"Services Menu title");
   NSMenu *menuServices = [[[NSMenu alloc] initWithTitle: title] autorelease];
-  NSString *appname = [[[[NSBundle mainBundle] localizedInfoDictionary] objectForKey:@"CFBundleDisplayName"] capitalizedString];
-  if (appname == nil)
-    appname = [[[NSProcessInfo processInfo] processName] capitalizedString];
+  NSString *appname = get_application_name ();
   [NSApp setServicesMenu: menuServices];
 
   [app_menu addItem: [NSMenuItem separatorItem]];
@@ -295,16 +338,26 @@ gtkosx_application_constructor (GType gtype,
                                 GObjectConstructParam *properties)
 {
   static GObject *self = NULL;
+#if GLIB_CHECK_VERSION (2,32,0)
+  static GMutex mutex;
+  g_mutex_init (&mutex);
+  g_mutex_lock (&mutex);
+#else
   static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
   g_static_mutex_lock (&mutex);
+#endif
   if (self == NULL)
     {
       self = G_OBJECT_CLASS (gtkosx_application_parent_class)->constructor (gtype, n_properties, properties);
       g_object_add_weak_pointer (self, (gpointer) &self);
 
     }
+#if GLIB_CHECK_VERSION (2,32,0)
+  g_mutex_unlock (&mutex);
+  g_mutex_clear (&mutex);
+#else
   g_static_mutex_unlock (&mutex);
-
+#endif
   return g_object_ref (self);
 
 }
@@ -439,19 +492,28 @@ global_event_filter_func (gpointer  windowing_event, GdkEvent *event,
   GtkosxApplication* app = user_data;
 
   /* Handle menu events with no window, since they won't go through
-   * the regular event processing. We have to release the gdk mutex so
-   * that we can recquire it when we invoke the gtk handler. Note well
-   * that handlers need to wrap any calls into gtk in
-   * gdk_threads_enter() and gdk_threads_leave() in a multi-threaded
-   * environment!
+   * the regular event processing. When using Gtk before 3.6, we have
+   * to release the gdk mutex so that we can recquire it when we
+   * invoke the gtk handler. Note well that handlers need to wrap any
+   * calls into gtk in gdk_threads_enter() and gdk_threads_leave() in
+   * a multi-threaded environment!
+   *
+   * With Gtk 3.6 and later, all Gtk calls must be made from the main
+   * thread, so be sure that handlers aren't invoked on a worker
+   * thread; use an idle or timer event to start the handler if
+   * necessary.
    */
   if ([nsevent type] == NSKeyDown &&
       gtkosx_application_use_quartz_accelerators (app) )
     {
       gboolean result;
+#if GTK_CHECK_VERSION (3, 6, 0)
+      result = [[NSApp mainMenu] performKeyEquivalent: nsevent];
+#else
       gdk_threads_leave ();
       result = [[NSApp mainMenu] performKeyEquivalent: nsevent];
       gdk_threads_enter ();
+#endif
       if (result) return GDK_FILTER_TRANSLATE;
     }
   return GDK_FILTER_CONTINUE;
@@ -749,12 +811,10 @@ gtkosx_application_insert_app_menu_item (GtkosxApplication* self,
     zero'th item, it's the About item. */
   if (index == 0)
     {
-      gchar *label = gtk_menu_item_get_label (GTK_MENU_ITEM (item));
-      gchar *appname;
-      NSString *nsappname = [[[NSBundle mainBundle] localizedInfoDictionary] objectForKey:@"CFBundleDisplayName"];
-      if (nsappname == nil)
-	nsappname = [[NSProcessInfo processInfo] processName];
-      appname = [[nsappname capitalizedString] UTF8String];
+      GtkWidget *widgetLabel = NULL;
+      const gchar *label = get_menu_label_text (item, &widgetLabel);
+      NSString *nsappname = get_application_name ();
+      const gchar *appname = [nsappname UTF8String];
       gtk_menu_item_set_label (GTK_MENU_ITEM (item), g_strdup_printf ("%s %s", label, appname));
     }
 
