@@ -1,25 +1,26 @@
-/ *   W i n d o w s | B r i n g   A l l   T o   F r o n t   m e n u   i t e m   t i t l e   * / 
- " B r i n g   A l l   t o   F r o n t "   =   " W s z y s t k i e   w p r o w a d z a j   d o   p r z o d u " ; 
+ / *   W i n d o w s | B r i n g   A l l   T o   F r o n t   m e n u   i t e m   t i t l e   * / 
+ " B r i n g   A l l   t o   F r o n t "   =   " P r z y n i e[   w s z y s t k o   n a   w i e r z c h " ; 
  
  / *   H i d e   m e n u   i t e m   t i t l e   * / 
- " H i d e "   =   " U k r y " ; 
+ " H i d e "   =   " U k r y " ; 
  
  / *   H i d e   O t h e r s   m e n u   i t e m   t i t l e   * / 
  " H i d e   O t h e r s "   =   " U k r y j   I n n e " ; 
  
  / *   W i n d o w s | M i n i m i z e   m e n u   i t e m   * / 
- " M i n i m i z e "   =   " M Z m i n i m a l i z o w a " ; 
+ " M i n i m i z e "   =   " Z m i n i m a l i z u j " ; 
  
  / *   Q u i t   m e n u   i t e m   t i t l e   * / 
  " Q u i t "   =   " Z a m k n i j " ; 
  
  / *   S e r v i c e s   M e n u   I t e m   t i t l e   * / 
- " S e r v i c e s "   =   " U s Bu g i " ; 
+ " S e r v i c e s "   =   " U sB u g i " ; 
  
  / *   S h o w   A l l   m e n u   i t e m   t i t l e   * / 
- " S h o w   A l l "   =   " P o k a |  w s z y s t k i e " ; 
+ " S h o w   A l l "   =   " P o k a|   w s z y s t k o " ; 
  
  / *   W i n d o w   M e n u   t i t l e   * / 
  " W i n d o w "   =   " O k n o " ; 
  
- 
\ No newline at end of file
+ / *   H e l p   M e n u   t i t l e   * / 
+ " H e l p "   =   " P o m o c " ; 
